import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import 'package:intl/intl.dart';
import 'package:loading_animation_widget/loading_animation_widget.dart';
import 'colors.dart';
import 'firebase_functions.dart';
import 'main.dart';

List<BlogData> adminBlogData = allBlogs;


///Blog page for admins or employees at Vitensenteret
class AdminBlogPage extends StatefulWidget {
  const AdminBlogPage({super.key});
  @override
  State<AdminBlogPage> createState() => _AdminBlogPageState();
}

class _AdminBlogPageState extends State<AdminBlogPage> {

  //Variables for database
  bool fetchedData = false;
  Future<void> fetchedDataCheck = firebaseGetUser();
  bool fetchedDataBlog = false;
  Future<void> fetchedDataCheckBlog = firebaseGetBlog();

  //Knowing which state to activate
  int selectedIndex = 0;
  bool selectBlogToDelete = false;


  //Variable for the form
  final _formKey = GlobalKey<FormState>();

  //Variables storing changes locally
  BlogData tempPostData = BlogData(content: "", image: "", notify: false, publicationDate: "", title: "", author: "");
  var content;
  var image;
  bool notify = false;
  var publicationDate;
  var title;
  var author;

  bool cantFindBlog = false;
  int indexOfEditedBlog = 0;
  int indexOfDeletedBlog = 0;
  late String idOfEditBlog;
  late String idOfDeleteBlog;

  //Images
  late String uniqueFileName;
  late Uint8List previewImage;
  bool imagePicked = false;
  String imageUrl = '';
  

  @override
  Widget build(BuildContext context) {
    // Get data from database
    if(!fetchedData){
      fetchedDataCheck.whenComplete(() => setState((){
        fetchedData = true;
      }));
    }

    if(!fetchedDataBlog){
      fetchedDataCheckBlog.whenComplete(() => setState((){
        fetchedDataBlog = true;
      }));
    }

    // When data is fetched, show content
    if (fetchedData && fetchedDataBlog){
      switch (selectedIndex){
        //Delete blog
        case 1: return Scaffold(
          body: Container(
            constraints: const BoxConstraints.expand(),
            color: blaa,
            child: Padding(
              padding: const EdgeInsets.all(15.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  SizedBox(
                    width: (MediaQuery.of(context).size.width)/2,
                    child: Form(
                      key: _formKey,
                      child: Column(
                        children: <Widget>[
                          TextFormField(
                              decoration: const InputDecoration(
                                hintText: 'Tittel på ønsket blogginlegg som skal slettes',
                              ),
                              style: const TextStyle(
                                  fontSize: 17
                              ),
                              validator: (String? value) {
                                if (value == null || value.isEmpty) {
                                  return 'Må ha tittel';
                                }
                                title = value;
                                return null;
                              }
                          ),
                          
                          const SizedBox(height: 30,),

                          TextButton(
                            style: raisedButtonStyle,
                            onPressed: () {
                              if (_formKey.currentState!.validate()) {
                                for(int i = 0; i < allBlogs.length; i++){
                                  if (title == allBlogs[i].title){ //If the blog is found
                                    setState(() {
                                      cantFindBlog = false;
                                      selectBlogToDelete = true;
                                      indexOfDeletedBlog = i;
                                      idOfDeleteBlog = allBlogs[i].blogID;
                                    });
                                  }
                                } if(selectBlogToDelete == false) {
                                  setState(() {
                                    cantFindBlog = true;
                                  });
                                }
                              }
                            },
                            child: const Text("Slett blogg innlegg", style: TextStyle(color: blaa)),
                          ),

                          if (cantFindBlog == true)
                            const Text("Dette innlegget eksisterer ikke, vær obs å skrive HELT likt som tittel"),

                          if(selectBlogToDelete)
                            SizedBox(
                              height: MediaQuery.of(context).size.height/10,
                            ),

                          if(selectBlogToDelete)
                            TextButton(
                              style: raisedButtonStyleRed,
                              onPressed: () {
                                //Update data in database
                                Future<bool> delete = firebaseDeleteBlog(allBlogs[indexOfDeletedBlog]);

                                delete.whenComplete(() => setState(() {
                                  selectedIndex = 0;
                                  indexOfDeletedBlog = 0;
                                }),
                                );
                              },
                              child: const Text('Slett Blogg, Kan ikke angres!', style: TextStyle(color: blaa)),
                            ),

                          SizedBox(
                            height: MediaQuery.of(context).size.height/10,
                          ),

                          TextButton(
                            style: raisedButtonStyleRed,
                            onPressed: () =>
                                setState(() {
                                  selectedIndex = 0;
                                }),
                            child: const Text('Admin blogg side', style: TextStyle(color: blaa)),
                          ),
                        ],
                      ),
                    ),
                  ),

                ],
              ),
            ),
          ),
        );

        //Edit selected Blog
        case 2: return Scaffold(
          body: Container(
            constraints: const BoxConstraints.expand(),
            color: blaa,
            child: SingleChildScrollView(
              physics: const AlwaysScrollableScrollPhysics(),
              child: Padding(
                  padding: const EdgeInsets.all(15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      SizedBox(
                        width: (MediaQuery.of(context).size.width)/2,
                        child: Form(
                          key: _formKey,
                          child: Column(
                            children: <Widget>[

                              TextFormField(
                                  initialValue: allBlogs[indexOfEditedBlog].title,
                                  style: const TextStyle(
                                    fontSize: 25,
                                    fontWeight: FontWeight.bold,
                                  ),
                                  validator: (String? value) {
                                    if (value == null || value.isEmpty) {
                                      return 'Må ha tittel';
                                    }
                                    title = value;
                                    return null;
                                  }
                              ),

                              const SizedBox(height: 30,),

                              TextFormField(
                                initialValue: allBlogs[indexOfEditedBlog].content,

                                style: const TextStyle(
                                  fontSize: 25,
                                  fontWeight: FontWeight.bold,
                                ),
                                validator: (String? value) {
                                  if (value == null || value.isEmpty) {
                                    return 'Må ha innhold';
                                  }
                                  content = value;
                                  return null;
                                },
                              ),

                              const SizedBox(height: 30,),

                              TextButton(
                                style: raisedButtonStyle,
                                onPressed: () {
                                  if (_formKey.currentState!.validate()) {

                                    if (title != "") {allBlogs[indexOfEditedBlog].title = title;}
                                    if (content != "") {allBlogs[indexOfEditedBlog].content = content;}

                                    //Update data in database
                                    Future<bool> update = firebaseUpdateBlog(allBlogs[indexOfEditedBlog]);

                                    update.whenComplete(() => setState(() {
                                      selectedIndex = 0;
                                      indexOfEditedBlog = 0;
                                    }),
                                    );
                                  }
                                },
                                child: const Text('Oppdater', style: TextStyle(color: blaa)),
                              ),

                              SizedBox(
                                height: MediaQuery.of(context).size.height/10,
                              ),

                              TextButton(
                                style: raisedButtonStyleRed,
                                onPressed: () =>
                                    setState(() {
                                      selectedIndex = 0;
                                    }),
                                child: const Text('Admin blogg side', style: TextStyle(color: blaa)),
                              ),
                            ],
                          ),
                        ),
                      ),
                    ],
                  )
              ),
            ),
          ),
        );

        //Edit blog
        case 3: return Scaffold(
          body: Container(
            constraints: const BoxConstraints.expand(),
            color: blaa,
            child: SingleChildScrollView(
              physics: const AlwaysScrollableScrollPhysics(),
              child: Padding(
                padding: const EdgeInsets.all(15.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    SizedBox(
                      width: (MediaQuery.of(context).size.width)/2,
                      child: Form(
                        key: _formKey,
                        child: Column(
                          children: <Widget>[
                            TextFormField(
                                decoration: const InputDecoration(
                                  hintText: 'Tittel på ønsket blogginlegg',
                                ),
                                style: const TextStyle(
                                    fontSize: 17
                                ),
                                validator: (String? value) {
                                  if (value == null || value.isEmpty) {
                                    return 'Må ha tittel';
                                  }
                                  title = value;
                                  return null;
                                }
                            ),

                            const SizedBox(height: 30,),

                            TextButton(
                              style: raisedButtonStyle,
                              onPressed: () {
                                if (_formKey.currentState!.validate()) {
                                  for(int i = 0; i < allBlogs.length; i++){
                                    if (title == allBlogs[i].title){ //If the blog is found
                                      setState(() {
                                        cantFindBlog = false;
                                        selectedIndex = 2;
                                        indexOfEditedBlog = i;
                                      });
                                    }
                                  }
                                  setState(() {
                                    cantFindBlog = true;
                                  });
                                }
                              },
                              child: const Text('Finn Blogg Innlegg', style: TextStyle(color: blaa)),
                            ),

                            if (cantFindBlog)
                              const Text("Dette innlegget eksisterer ikke, vær obs å skrive HELT likt som tittel"),

                            SizedBox(
                              height: MediaQuery.of(context).size.height/10,
                            ),

                            TextButton(
                              style: raisedButtonStyleRed,
                              onPressed: () =>
                                  setState(() {
                                    selectedIndex = 0;
                                  }),
                              child: const Text('Admin blogg side', style: TextStyle(color: blaa)),
                            ),
                          ],
                        ),
                      ),
                    ),

                  ],
                ),
              ),
            ),
          ),
        );

        //Add blog
        case 4: return Scaffold(
          body: Container(
            constraints: const BoxConstraints.expand(),
            color: blaa,
            child: SingleChildScrollView(
              physics: const AlwaysScrollableScrollPhysics(),
              child: Padding(
                padding: const EdgeInsets.all(15.0),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    SizedBox(
                      width: (MediaQuery
                          .of(context)
                          .size
                          .width) / 2,
                      child: Form(
                        key: _formKey,
                        child: Column(
                          children: <Widget>[

                            TextFormField(
                                decoration: const InputDecoration(
                                  hintText: 'Tittel',
                                ),
                                style: const TextStyle(
                                    fontSize: 17
                                ),
                                validator: (String? value) {
                                  if (value == null || value.isEmpty) {
                                    return 'Må ha tittel';
                                  }
                                  title = value;
                                  return null;
                                }
                            ),

                            const SizedBox(height: 30,),

                            TextFormField(
                              decoration: const InputDecoration(
                                hintText: 'Innhold av blogg',
                              ),
                              style: const TextStyle(
                                  fontSize: 17
                              ),
                              validator: (String? value) {
                                if (value == null || value.isEmpty) {
                                  return 'Må ha innhold';
                                }
                                content = value;
                                return null;
                              },
                            ),

                            const SizedBox(height: 30,),

                            if(imagePicked)Image.memory(previewImage, fit: BoxFit.cover),

                            const SizedBox(height: 30,),

                            if(imageUrl.isEmpty)TextButton(
                              style: raisedButtonStyle,
                              onPressed: () async {
                                ImagePicker imagePicker = ImagePicker();
                                XFile? file = await imagePicker.pickImage(
                                    source: ImageSource.gallery);
                                uniqueFileName = DateTime
                                    .now()
                                    .millisecondsSinceEpoch
                                    .toString();

                                if (file == null) return;

                                var f = await file.readAsBytes();
                                setState(() {
                                  imagePicked = true;
                                  previewImage = f;
                                });
                              },
                              child: Text((imagePicked) ? "Bytt bilde" : "Legg til bilde",
                                  style: const TextStyle(color: blaa)),
                            ),


                            const Text("Lag en notifikasjon ved opplastning?"),

                            Checkbox(
                              checkColor: Colors.white,
                              value: notify,
                              onChanged: (bool? value) {
                                setState(() {
                                  notify = value!;
                                });
                              },
                            ),

                            const SizedBox(height: 30,),

                            TextButton(
                              style: raisedButtonStyle,
                              onPressed: () {
                                if (_formKey.currentState!.validate()) {
                                  //Upload image
                                  Reference referenceRoot = FirebaseStorage.instance.ref();
                                  Reference referenceDirImages = referenceRoot.child(
                                      '/blogPhotos/$uniqueFileName');
                                  Future<bool> upload = firebaseUploadImageForBlog(
                                      previewImage, uniqueFileName);
                                  upload.whenComplete(() =>
                                      setState(() async {
                                        imageUrl =
                                        await referenceDirImages.getDownloadURL();
                                        tempPostData.image = imageUrl;
                                        tempPostData.title = title;
                                        tempPostData.content = content;
                                        tempPostData.notify = notify;


                                        //Auto generated data
                                        tempPostData.author = currentUser.name;
                                        var now = DateTime.now();
                                        var formatter = DateFormat('dd-MM-yyyy');
                                        String formattedDate = formatter.format(now);
                                        tempPostData.publicationDate = formattedDate;

                                        //Update data in database
                                        Future<bool> update = firebaseUploadBlog(
                                            tempPostData);

                                        update.whenComplete(() =>
                                            setState(() {
                                              selectedIndex = 0;
                                            }),
                                        );
                                      }),
                                  );
                                }
                              },
                              child: const Text("Last opp", style: TextStyle(color: blaa)),
                            ),
                            SizedBox(
                              height: MediaQuery
                                  .of(context)
                                  .size
                                  .height / 10,
                            ),
                            TextButton(
                              style: raisedButtonStyleRed,
                              onPressed: () =>
                                  setState(() {
                                    selectedIndex = 0;
                                  }),
                              child: const Text('Admin blogg side',
                                  style: TextStyle(color: blaa)),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        );

        //No page selected
        default: return Center(
            child: SizedBox.expand(
              child: Container(
                color: blaa,
                child: Column(
                  children: [

                    SizedBox(
                      height: MediaQuery.of(context).size.height/10,
                    ),

                    TextButton(
                      style: raisedButtonStyle,
                      onPressed: () {setState(() {
                        selectedIndex = 4;
                      });},
                      child: const Text('Opprett innlegg', style: TextStyle(color: blaa)),
                    ),

                    SizedBox(
                      height: MediaQuery.of(context).size.height/10,
                    ),

                    TextButton(
                      style: raisedButtonStyle,
                      onPressed: () {
                        setState(() {
                          selectedIndex = 3;
                        });
                      },
                      child: const Text('Rediger innlegg', style: TextStyle(color: blaa)),
                    ),

                    SizedBox(
                      height: MediaQuery.of(context).size.height/10,
                    ),

                    TextButton(
                      style: raisedButtonStyle,
                      onPressed: () {
                        setState(() {
                          selectedIndex = 1;
                        });
                      },
                      child: const Text('Slett innlegg', style: TextStyle(color: blaa)),
                    ),

                    SizedBox(
                      height: MediaQuery.of(context).size.height/10,
                    ),

                    TextButton(
                      style: raisedButtonStyleRed,
                      onPressed: () => Navigator.push(
                        context,
                        MaterialPageRoute(builder: (context) => const MyApp()),
                      ),
                      child: const Text('Blogg oversikt', style: TextStyle(color: blaa)),
                    ),

                    SizedBox(
                      height: MediaQuery.of(context).size.height/10,
                    ),
                  ],
                ),
              ),
            )
        );
      }
      
    }else{
      //Loading
      return SizedBox.expand( //Loading screen until data is fetched from firebase
        child: Container(
          color: blaa,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              LoadingAnimationWidget.staggeredDotsWave(
                size: 100,
                color: morkBlaa,
              ),
            ],
          ),
        ),
      );
    }
  }
}




