import 'package:billettapp_for_vitensenteret/programmingGame/programmingGame.dart';
import 'package:billettapp_for_vitensenteret/towerOfHanoi/tower_of_hanoi.dart';
import 'package:flutter/material.dart';
import 'package:billettapp_for_vitensenteret/wordle/wordle.dart';
import 'colors.dart';

/// Navigation widget, redirects to [GameMenuPage]
class GamePage extends StatelessWidget {
  const GamePage({super.key});

  @override
  Widget build(BuildContext context) {
    return Navigator(
      onGenerateRoute: (settings) {
        return MaterialPageRoute(builder: (_) => const GameMenuPage());
      },
    );
  }
}

/// Page with all games available, allows a user to choose a game to play
class GameMenuPage extends StatefulWidget {
  const GameMenuPage({Key? key}) : super(key: key);

  @override
  _GameMenuPageState createState() => _GameMenuPageState();
}

class _GameMenuPageState extends State<GameMenuPage> {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: blaa,
      body: SafeArea(
          child: Column(children: [

            // Header
            Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: const [
                  Text(
                    'Spillgalleri:',
                    style: TextStyle(
                      fontSize: 36,
                      color: morkBlaa,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ]),

            Expanded(
                child: ListView(
              physics: const BouncingScrollPhysics(),
              children: [

                //Image of VI logo
                Center(
                  child: Image.asset( //Logo of Vitensenteret
                    'assets/images/VITENSENTERET_horisontal_blaa.png',
                    height: MediaQuery.of(context).size.height/2,
                    width: MediaQuery.of(context).size.width/2,
                  ),
                ),

                // menu with all available games
                Padding(
                  padding: const EdgeInsets.all(5.0),
                  child: Row( //The following shows all available games with their logos and names. they are clickable and take you to their games.
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      _cardMenu(
                        onTap: () {
                          Navigator.push(
                            context,
                            MaterialPageRoute(builder: (context) => const Wordle()),
                          );
                        },
                        title: 'Viten Ord',
                        icon: 'assets/images/vitenOrd.png',
                      ),_cardMenu(
                        onTap: () {
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => const TeachProgrammingGame()),
                          );
                        },
                        title: 'Viten Kode',
                        icon: 'assets/images/ProgSpill.png',
                      ),
                      _cardMenu(
                        onTap: () {
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => const HannoisTower()),
                          );
                        },
                        title: 'Hanois Tårn',
                        icon: 'assets/images/TowerGame.png',
                      ),
                    ],
                  ),
                ),
              ],
            ))
          ])),
    );
  }

  /// Makes the logos look nice inside their own light box, that is clickable
  Widget _cardMenu({
    required String title,
    required String icon,
    VoidCallback? onTap,
    Color color = Colors.white,
  }) {
    return GestureDetector(
        onTap: onTap,
        child: Container(
            width: MediaQuery.of(context).size.width/5,
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(16),
              color: color,
            ),
            child: Column(children: [
              Image.asset(icon, height: MediaQuery.of(context).size.height/6, width: MediaQuery.of(context).size.width/6),
              Text(title,
                  style: const TextStyle(
                    fontWeight: FontWeight.bold,
                    color: morkBlaa,
                  ))
            ])));
  }
}

/// Wordle game
/// Note that a lot of the code present in the Wordle game is inspired from this
/// tutorial located at: https://www.youtube.com/watch?v=_W0RN_Cqhpg&ab_channel=MarcusNg
class Wordle extends StatelessWidget {
  const Wordle({super.key});

  @override
  Widget build(BuildContext context) {
    return const MaterialApp(
      title: 'Flutter Wordle App',
      debugShowCheckedModeBanner:false,
      home: WordleScreen(),
    );
  }
}

/// Tower of hanoi, rules can be found here:
/// https://en.wikipedia.org/wiki/Tower_of_Hanoi
class HannoisTower extends StatelessWidget {
  const HannoisTower({super.key});

  @override
  Widget build(BuildContext context) {
    return const MaterialApp(
      title: 'Flutter HannoisTower App',
      debugShowCheckedModeBanner: false,
      home: TowerOfHanoi(),
    );
  }
}

/// Programming game
class TeachProgrammingGame extends StatelessWidget {
  const TeachProgrammingGame({super.key});

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Coding App',
      debugShowCheckedModeBanner: false,
      theme: ThemeData.dark().copyWith(scaffoldBackgroundColor: Colors.black),
      home: const ProgrammingGame(),
    );
  }
}
