import 'dart:convert';
import 'dart:developer';
import 'package:billettapp_for_vitensenteret/firebase_functions.dart';
import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'package:flutter/material.dart';
import 'package:flutter_stripe/flutter_stripe.dart';
import 'package:http/http.dart' as http;
import 'package:loading_animation_widget/loading_animation_widget.dart';
import 'colors.dart';

/// Payment page widget
class PaymentPage extends StatefulWidget{
  const PaymentPage({super.key});

  @override
  State<PaymentPage> createState() => _PaymentPageState();
}

class _PaymentPageState extends State<PaymentPage> {
  late TextEditingController _giftCodeController;

  bool u = false;
  Future<void> loadedUser = firebaseGetUser();
  String giftCode = "";

  @override
  void initState() {
    super.initState();
    _giftCodeController = TextEditingController();
  }

  @override
  void dispose() {
    _giftCodeController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context){

    // Ensure user data is fetched
    loadedUser.whenComplete(() => setState(() {
      u = true;
    }));

    if(u) {
      return Scaffold(
        appBar: AppBar(
          title: const Text("Ny billett"),
        ),
        body: Padding(
          padding: const EdgeInsets.all(15.0),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: [

              // Button for buying tickets
              ElevatedButton(
                  onPressed: () async {
                    //await initPayment(context: context);
                    Navigator.push(context, MaterialPageRoute(builder: (context) => const ChooseTickets()));
                  },

                  child: const Text(
                    "Kjøp billett",
                    style: TextStyle(
                      fontSize: 20
                    ),
                  )
              ),

              // Button for activating a gift card
              ElevatedButton(
                  onPressed: () {
                    _giftCard(context);
                  },
                  child: const Text(
                    "Løs inn gavekortkode",
                    style: TextStyle(
                      fontSize: 20
                    ),
                  )
              ),
            ],
          ),
        ),
      );
    }else{
      return SizedBox.expand( //Loading screen until data is fetched from firebase
        child: Container(
          color: blaa,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              LoadingAnimationWidget.staggeredDotsWave(
                size: 100,
                color: morkBlaa,
              ),
            ],
          ),
        ),
      );
    }
  }

  /// Pop-up for activating gift cards, uses [context] from parent
  _giftCard(BuildContext context) {
    showDialog(
      context: context,
      builder: (context) {
        return StatefulBuilder(
            builder: (context, setState) {
          return AlertDialog(
            title: const Text("Løs inn gavekort"),
            content: SingleChildScrollView(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  TextField(
                    controller: _giftCodeController,
                    decoration: const InputDecoration(
                        border: OutlineInputBorder(),
                        labelText: "Gavekortkode"
                    ),
                    onChanged: (value) {
                      giftCode = value;
                    },
                  ),
                  ElevatedButton(
                    onPressed: () {
                      //TODO: CAll firebase function to activate gift card
                    },
                    child: const Text("Aktiver"),
                  )
                ],
              ),
            ),
          );
        });
      }
    );
  }
}


/// Page for adding/subtracting tickets the user plans to buy
class ChooseTickets extends StatefulWidget{
  const ChooseTickets({super.key});

  @override
  State<ChooseTickets> createState() => _ChooseTicketsState();
}

class _ChooseTicketsState extends State<ChooseTickets> {

  //Ticket amounts
  int countAdultTickets = 0;
  int countChildTickets = 0;
  int countFamilyTickets = 0;
  int countYearTickets = 0;

  //Prices of tickets (100 = 1 NOK)
  int priceAdultTickets = 11000;
  int priceChildTickets = 11000;
  int priceFamilyTickets = 37000;
  int priceYearTickets = 45000;

  //Sum of prices
  int sum = 0;

  //Default date selection for tickets
  DateTime selectedDate = DateTime.now();

  /// Initializes payment for the user, uses [context] from parent
  /// Function fetched from https://www.youtube.com/watch?v=BR4sF_VzV0w
  /// Github: https://github.com/hadikachmar3/grocery_app_course
  Future<void> initPayment({required BuildContext context})async{
    String currentDate = DateTime.now().year.toString() + ((DateTime.now().month.toString().length == 1)? "0": "") + DateTime.now().month.toString() +
        ((DateTime.now().day.toString().length == 1)? "0": "") + DateTime.now().day.toString();
    String selectedDateStart = selectedDate.year.toString() + ((selectedDate.month.toString().length == 1)? "0": "") + selectedDate.month.toString() +
        ((selectedDate.day.toString().length == 1)? "0": "") + selectedDate.day.toString();
    String nextyeardate = (selectedDate.year + 1).toString() + ((selectedDate.month.toString().length == 1)? "0": "") + selectedDate.month.toString() +
        ((selectedDate.day.toString().length == 1)? "0": "") + selectedDate.day.toString();
    List <TicketData> tickets = [];

    //Add tickets to ticket list
    for(var i = 0; i < countAdultTickets; i++){
      var ticket = TicketData(
          activatedBy: currentUser.name,
          activationDate: selectedDateStart,
          endDate: selectedDateStart,
          purchaseDate: currentDate,
          purchasedBy: currentUser.name,
          giftCode: "code",
          type: "Voksenbillett");
      tickets.add(ticket);
    }
    for(var i = 0; i < countChildTickets; i++){
      var ticket = TicketData(
          activatedBy: currentUser.name,
          activationDate: selectedDateStart,
          endDate: selectedDateStart,
          purchaseDate: currentDate,
          purchasedBy: currentUser.name,
          giftCode: "code",
          type: "Barnebillett");
      tickets.add(ticket);
    }
    for(var i = 0; i < countFamilyTickets; i++){
      var ticket = TicketData(
          activatedBy: currentUser.name,
          activationDate: selectedDateStart,
          endDate: selectedDateStart,
          purchaseDate: currentDate,
          purchasedBy: currentUser.name,
          giftCode: "code",
          type: "Familiebillett");
      tickets.add(ticket);
    }
    for(var i = 0; i < countYearTickets; i++){
      var ticket = TicketData(
          activatedBy: currentUser.name,
          activationDate: selectedDateStart,
          endDate: nextyeardate,
          purchaseDate: currentDate,
          purchasedBy: currentUser.name,
          giftCode: "code",
          type: "Årskort");
      tickets.add(ticket);
    }

    // Format userdata for firebase function
    var user = UserData(
        birthday: currentUser.birthday,
        email: currentUser.email,
        image: currentUser.image,
        isAdmin: currentUser.isAdmin,
        isEmployee: currentUser.isEmployee,
        name: currentUser.name,
        phone: currentUser.phone,
        tickets: currentUser.tickets);

    try{
      //Update sum
      calculateSum();

      // Create payment sheet
      final response = await http.post(
          Uri.parse(
              "https://europe-west1-mockaarskort.cloudfunctions.net/stripePaymentIntentRequest"),
          body:{
            'email': currentUser.email,
            'amount': sum.toString(),
          });
      // Decode and read response
      final jsonResponse = jsonDecode(response.body);
      log(jsonResponse.toString());

      // Initialize the payment sheet
      await Stripe.instance.initPaymentSheet(
          paymentSheetParameters: SetupPaymentSheetParameters(
            paymentIntentClientSecret: jsonResponse['paymentIntent'],
            merchantDisplayName: 'Vitensenteret Innlandet',
            customerId: jsonResponse['customer'],
            customerEphemeralKeySecret: jsonResponse['ephemeralKey'],
          ));
      await Stripe.instance.presentPaymentSheet();


      //Payment success
      if (await firebaseAddMultipleTicketsToDatabase(tickets, user, currentUser.userID)){
        ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text("Payment complete")));
      }else{
        ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text("Payment went through but database couldn't be updated")));
      }


      //Error handling
    }catch(err){
      //Stripe errors
      if(err is StripeException){
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text("Payment error ${err.error.localizedMessage}")));
        //Other errors
      }else{
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text("Payment error $err")));
      }
    }
  }

  /// Sums the price of all tickets
  void calculateSum(){
    sum = countAdultTickets*priceAdultTickets +
          countChildTickets*priceChildTickets +
          countFamilyTickets*priceFamilyTickets +
          countYearTickets*priceYearTickets;
  }

  /// Function for picking a date, gathered from:
  /// https://www.appsdeveloperblog.com/datepicker-example-in-flutter/
  Future<void> _selectDate(BuildContext context) async {
    final DateTime? pickedDate = await showDatePicker(
      context: context,
      initialDate: DateTime.now(),
      firstDate: DateTime.now(),
      lastDate: DateTime(DateTime.now().year + 1, DateTime.now().month, DateTime.now().day),
    );
    if (pickedDate != null) {
      setState(() {
        selectedDate = pickedDate;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    calculateSum();
    return Scaffold(
      appBar: AppBar(
        title: const Text("Kjøp billett"),
      ),
      body: Padding(
        padding: const EdgeInsets.only(top: 40, left: 8, right: 8),
        child: Column(
          children: [
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [

                // Each row consists of type of ticket, and buttons for adding or subtracting tickets of that type
                const Text(
                  "Voksenbillett:",
                  style: TextStyle(
                    fontSize: 20,
                    fontWeight: FontWeight.bold
                  ),
                ),
                Row(
                  children: [
                    ElevatedButton(
                      onPressed: () {
                        if (countAdultTickets > 0) {
                          setState(() {
                            countAdultTickets--;
                          });
                        }
                      },
                      child: const Text("-"),
                    ),
                    ElevatedButton(
                      onPressed: () {

                      },
                      child: Text(countAdultTickets.toString()),
                    ),
                    ElevatedButton(
                      onPressed: () {
                        if (countAdultTickets < 50) {
                          setState(() {
                            countAdultTickets++;
                          });
                        }
                      },
                      child: const Text("+"),
                    ),
                  ],
                ),
              ],
            ),

            const SizedBox(height: 10,),

            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                const Text(
                  "Barnebillett (4-16):",
                  style: TextStyle(
                      fontSize: 20,
                      fontWeight: FontWeight.bold
                  ),
                ),
                Row(
                  children: [
                    ElevatedButton(
                      onPressed: () {
                        if (countChildTickets > 0) {
                          setState(() {
                            countChildTickets--;
                          });
                        }
                      },
                      child: const Text("-"),
                    ),
                    ElevatedButton(
                      onPressed: () {

                      },
                      child: Text(countChildTickets.toString()),
                    ),
                    ElevatedButton(
                      onPressed: () {
                        if (countChildTickets < 50) {
                          setState(() {
                            countChildTickets++;
                          });
                        }
                      },
                      child: const Text("+"),
                    ),
                  ],
                ),
              ],
            ),

            const SizedBox(height: 10,),

            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                const Text(
                  "Familiebillett \n(2 voksne, 2 barn):",
                  style: TextStyle(
                      fontSize: 20,
                      fontWeight: FontWeight.bold
                  ),
                ),
                Row(
                  children: [
                    ElevatedButton(
                      onPressed: () {
                        if (countFamilyTickets > 0) {
                          setState(() {
                            countFamilyTickets--;
                          });
                        }
                      },
                      child: const Text("-"),
                    ),
                    ElevatedButton(
                      onPressed: () {

                      },
                      child: Text(countFamilyTickets.toString()),
                    ),
                    ElevatedButton(
                      onPressed: () {
                        if (countFamilyTickets < 50) {
                          setState(() {
                            countFamilyTickets++;
                          });
                        }
                      },
                      child: const Text("+"),
                    ),
                  ],
                ),
              ],
            ),

            const SizedBox(height: 10,),

            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                const Text(
                  "Årskort:",
                  style: TextStyle(
                      fontSize: 20,
                      fontWeight: FontWeight.bold
                  ),
                ),
                Row(
                  children: [
                    ElevatedButton(
                      onPressed: () {
                        if (countYearTickets > 0) {
                          setState(() {
                            countYearTickets--;
                          });
                        }
                      },
                      child: const Text("-"),
                    ),
                    ElevatedButton(
                      onPressed: () {

                      },
                      child: Text(countYearTickets.toString()),
                    ),
                    ElevatedButton(
                      onPressed: () {
                        if (countYearTickets < 50) {
                          setState(() {
                            countYearTickets++;
                          });
                        }
                      },
                      child: const Text("+"),
                    ),
                  ],
                ),
              ],
            ),

            // Divider line
            const Divider(
              thickness: 5,
            ),

            //Pick birth date
            Padding(
              padding: const EdgeInsets.only(top: 8.0),
              child: Row(
                children: [
                  const Icon(Icons.date_range, color: Colors.grey,),
                  const Padding(
                    padding: EdgeInsets.only(left: 15.0, right: 8.0),
                    child: Text(
                      "Dato:",
                      style: TextStyle(
                        fontSize: 17,
                        color: Colors.blueGrey,
                      ),),
                  ),
                  OutlinedButton(
                      onPressed: () {
                        _selectDate(context);
                      },
                      child: Text(
                        "${selectedDate.day}/${selectedDate.month}/${selectedDate.year}",
                        style: const TextStyle(
                          fontSize: 17,
                        ),
                      )
                  ),
                ],
              ),
            ),

            const SizedBox(height: 10,),

            // Sum
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                const Text(
                  "Sum:",
                  style: TextStyle(
                    fontSize: 20,
                    fontWeight: FontWeight.bold,
                  ),
                ),
                Text(
                  "${(sum/100).toStringAsFixed(2)} kr",
                  style: const TextStyle(
                    fontSize: 20,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ],
            ),

            const SizedBox(height: 10,),

            ElevatedButton(
              onPressed: () async {
                await initPayment(context: context);
              },
              child: const Text("Kjøp"),)
          ]
        ),
      ),
    );
  }
}
