import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'package:billettapp_for_vitensenteret/resources/util.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:billettapp_for_vitensenteret/firebase_functions.dart';
import 'package:image_picker/image_picker.dart';
import 'dart:io';
import 'authentication.dart';

///Redirects user to the settings page
class ProfilePage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    Widget page = const ProfilePageSettings();
    return Navigator(
      onGenerateRoute: (settings) {
        return MaterialPageRoute(builder: (_)=> page);
      },
    );
  }

}

///Settings page, contains the option to change some user data, log out and delete user
class ProfilePageSettings extends StatefulWidget {
  const ProfilePageSettings({super.key});

  @override
  State<ProfilePageSettings> createState() => _ProfilePageSettingsState();
}

class _ProfilePageSettingsState extends State<ProfilePageSettings> {
  final ImagePicker _picker = ImagePicker();
  final newEmailController = TextEditingController();
  final newPhoneController = TextEditingController();
  final oldPasswordController = TextEditingController();
  final newPasswordController = TextEditingController();
  final repeatPasswordController = TextEditingController();

  late File newProfilePic;
  late Uint8List webProfilePic;
  String newMail = "";
  String newPhone = "";
  String oldPassword = "";
  String newPassword = "";
  String repeatPassword = "";

  bool u = false;               //When true shows the page

  bool selectedMenu = false;

  @override
  void dispose() {
    newEmailController.dispose();
    newPhoneController.dispose();
    newPasswordController.dispose();
    oldPasswordController.dispose();
    repeatPasswordController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    //If someone is logged in
    if (FirebaseAuth.instance.currentUser != null) {

      //Get user info
      Future<void> loadedUser = firebaseGetUser();
      loadedUser.whenComplete(() => setState(() {
        u = true;
      }));

      var profilePic = currentUser.image;
      var screenHeight = MediaQuery.of(context).size.height;
      var screenWidth = MediaQuery.of(context).size.width;

      //If user has loaded in, show content
      if (u) {
        return Padding(
        padding: EdgeInsets.only(top: screenHeight/25, bottom: screenHeight/60, left: screenWidth/30, right: screenWidth/30),
        child: Column(
          children: [
            Expanded(
              child: Card(
                child: Column(
                  mainAxisSize: MainAxisSize.max,
                  children: [

                    // Delete user button
                    Align(
                      alignment: Alignment.centerRight,
                      child: TextButton(
                        onPressed: () {
                          //Delete user
                          firebaseDeleteUser(FirebaseAuth.instance.currentUser!.uid, true, true);
                          Navigator.push(context, MaterialPageRoute(builder: (context) => ProfilePage()));
                        },
                        child: const Text("Slett bruker",
                        style: TextStyle(
                          color: Colors.red,
                        ),),

                      )
                    ),

                    //Profile picture
                    CircleAvatar(
                      maxRadius: screenHeight/10,
                      backgroundImage: NetworkImage(
                        profilePic,
                      ),
                    ),

                    //Update profile picture button
                    TextButton(
                      style: TextButton.styleFrom(
                        textStyle: const TextStyle(fontSize: 15),
                        backgroundColor: Colors.white,
                      ),
                      onPressed: () async {
                        int uploadImageSuccess = 0;
                        //Use Image Picker and then upload the new image to Firebase Storage
                        if (!kIsWeb) {
                          final XFile? image = await _picker.pickImage(source: ImageSource.gallery);

                          if (image != null) {
                            newProfilePic = File(image.path);
                            uploadImageSuccess = await firebaseUpdateProfilePicture(newProfilePic);
                          }
                        }
                        else if (kIsWeb) {
                          XFile? image = await _picker.pickImage(source: ImageSource.gallery);

                          if (image != null){
                            var f = await image.readAsBytes();
                            webProfilePic = f;
                            //TODO: Upload images from web as well
                          }
                        }
                        //use feedback to alert users if errors happens
                        const snackBar = SnackBar(
                          content: Text('Problem under opplasting, prøv på nytt senere'),
                          backgroundColor: Colors.red,
                        );
                        if (uploadImageSuccess != 0) {
                          ScaffoldMessenger.of(context).showSnackBar(snackBar);
                        }
                      }, child: const Text("Oppdater profilbilde"),
                    ),

                    Expanded(
                      child: Padding(
                        padding: EdgeInsets.symmetric(vertical: screenHeight/50, horizontal: screenWidth/20),
                          child: Column(
                            mainAxisSize: MainAxisSize.max,
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [

                              //Show name
                              const Text("Navn",
                                style: TextStyle(
                                  fontSize: 15,
                                  color: Colors.grey
                                ),),
                              Padding(
                                padding: const EdgeInsets.only(bottom: 30, top: 0),
                                child: Text(currentUser.name,
                                  style: const TextStyle(
                                    fontSize: 25,
                                  ),),
                              ),

                              //Show email
                              const Text("Epost",
                                style: TextStyle(
                                  fontSize: 15,
                                  color: Colors.grey
                                ),),

                              //Change email button
                              TextButton(
                                  onPressed: () {
                                    _editMailDialogBuilder(context);
                                  },
                                style: TextButton.styleFrom(
                                  foregroundColor: Colors.blue,
                                  minimumSize: Size.zero,
                                  padding: EdgeInsets.zero,
                                  tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                                  textStyle: const TextStyle(
                                    fontSize: 20,
                                    fontWeight: FontWeight.bold,
                                  )
                                ),
                                child: Text(currentUser.email)
                              ),

                              //Empty space
                              SizedBox(height: screenHeight/40,),

                              //Show phone number
                              const Text("Mobilnummer",
                                style: TextStyle(
                                  fontSize: 15,
                                  color: Colors.grey
                                ),),

                              //Change phone number button
                              TextButton(
                                onPressed: () {
                                  _editPhoneDialogBuilder();
                                },
                                style: TextButton.styleFrom(
                                    foregroundColor: Colors.blue,
                                    //backgroundColor: Colors.blue,
                                    minimumSize: Size.zero,
                                    padding: EdgeInsets.zero,
                                    tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                                    textStyle: const TextStyle(
                                      fontSize: 20,
                                      fontWeight: FontWeight.bold,
                                    )
                                ),
                                child: Text(currentUser.phone),
                              ),

                              //Empty space
                              SizedBox(height: screenHeight/40,),

                              //Show 'Passord' text
                              const Text("Passord",
                                style: TextStyle(
                                  fontSize: 15,
                                  color: Colors.grey
                                ),
                              ),

                              //Change password button
                              TextButton(
                                  style: TextButton.styleFrom(
                                      foregroundColor: Colors.blue,
                                      //backgroundColor: Colors.blue,
                                      minimumSize: Size.zero,
                                      padding: EdgeInsets.zero,
                                      tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                                      textStyle: const TextStyle(
                                        fontSize: 20,
                                        fontWeight: FontWeight.bold,
                                      )
                                  ),
                                  onPressed: () {
                                    _editPasswordDialogBuilder();
                                  },
                                  child: const Text("Endre passord")
                              ),

                              const Spacer(),

                              //Log out button
                              Align(
                                alignment: FractionalOffset.bottomLeft,
                                child: TextButton(
                                  style: TextButton.styleFrom(
                                      foregroundColor: Colors.blue,
                                      //backgroundColor: Colors.blue,
                                      minimumSize: Size.zero,
                                      padding: EdgeInsets.zero,
                                      tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                                      textStyle: const TextStyle(
                                        fontSize: 20,
                                        fontWeight: FontWeight.bold,
                                      )
                                  ),
                                  onPressed: () async {
                                    await FirebaseAuth.instance.signOut();
                                    Navigator.push(context, MaterialPageRoute(builder: (context) => ProfilePage()));
                                  },
                                  child: const Text("Logg ut"),
                                ),
                              ),
                            ],
                          ),
                      ),
                    ),

                  ],
                ),
              ),
            ),

          ],
        ),
      );
      } else {
        //Empty space if user has not loaded in
        return const SizedBox(height: 2);
      }
    } else {
      //Button for logging in if the user is non-authenticated
      return Card(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            ElevatedButton(
                onPressed: () {
                  Navigator.push((context), MaterialPageRoute(builder: (context) => const LogInPage()));
                },
                child: const Text("Logg inn")
            )
          ],
        ),
      );
    }
  }

  /// Some code gathered from the accepted answer on this question:
  /// https://stackoverflow.com/questions/70529682/create-pop-up-dialog-in-flutter
  /// Pop-up window allowing a user to change their email, and uses [context] from where it is called
  _editMailDialogBuilder(BuildContext context) {
    showDialog (
      context: context,
      builder: (BuildContext context) {

        //Pop-up window
        return AlertDialog(
          title: const Text("Rediger epostadresse"),
          content: SingleChildScrollView(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[

                //Text input field for new email address
                Container(
                  padding: const EdgeInsets.all(8.0),
                  child: TextField(
                    controller: newEmailController,
                    onChanged: (String value) {
                      newMail = value;
                    },
                    decoration: const InputDecoration(
                        border: OutlineInputBorder(),
                        hintText: 'eksempel@domene.no',
                        labelText: 'Ny epostadresse'),
                  ),
                ),

                //Text input field for password, password is needed for
                //re-authentication before email can be changed
                Container(
                  padding: const EdgeInsets.all(8.0),
                  child: TextField(
                    obscureText: true,
                    controller: oldPasswordController,
                    onChanged: (String value) {
                      oldPassword = value;
                    },
                    decoration: const InputDecoration(
                        border: OutlineInputBorder(),
                        labelText: 'Passord'),
                  ),
                ),

                //Confirm update button
                Container(
                  width: double.infinity,
                  height: 60,
                  padding: const EdgeInsets.all(8.0),
                  child: ElevatedButton(
                    onPressed: () {
                      if (validateEmail(newMail)) {
                        //Errors are not handled
                        Future <int> update = firebaseUpdateMail(newMail, oldPassword);
                        update.whenComplete(() => Navigator.of(context).pop());
                      } else {
                        const snackBar = SnackBar(
                          content: Text('Feil format på epost'),
                          backgroundColor: Colors.red,
                        );
                        ScaffoldMessenger.of(context).showSnackBar(snackBar);
                        Navigator.of(context).pop();
                        }
                    },
                    child: const Text(
                      "Lagre",
                    ),
                  ),
                ),
              ],
            ),
          ),
        );
      }
    );
  }

  /// Some code gathered from the accepted answer on this question:
  /// https://stackoverflow.com/questions/70529682/create-pop-up-dialog-in-flutter
  /// Pop-up window allowing a user to change their phone number
  _editPhoneDialogBuilder() {
     showDialog (
        context: context,
        builder: (BuildContext context) {
          //Pop-up window
          return AlertDialog(
            title: const Text("Rediger mobilnummer"),
            content: SingleChildScrollView(
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[

                  //Text input field for phone number
                  Container(
                    padding: const EdgeInsets.all(8.0),
                    child: TextField(
                      controller: newPhoneController,
                      onChanged: (String value) {
                        newPhone = value;
                      },
                      keyboardType: const TextInputType.numberWithOptions(),
                      decoration: const InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Nytt mobilnummer'),
                    ),
                  ),

                  //Confirm update button
                  Container(
                    width: double.infinity,
                    height: 60,
                    padding: const EdgeInsets.all(8.0),
                    child: ElevatedButton(
                      onPressed: () {
                        if (validatePhone(newPhone)) {
                          //Errors are not handled
                          Future <int> update = firebaseUpdatePhone(newPhone);
                          update.whenComplete(() => Navigator.of(context).pop());
                        }
                        else {
                          const snackBar = SnackBar(
                            content: Text('Feil format på mobilnummer'),
                            backgroundColor: Colors.red,
                          );
                          ScaffoldMessenger.of(context).showSnackBar(snackBar);
                          Navigator.of(context).pop();
                        }
                      },
                      child: const Text(
                        "Lagre",
                      ),
                    ),
                  ),
                ],
              ),
            ),
          );
        }
    );
  }

  /// Some code gathered from the accepted answer on this question:
  /// https://stackoverflow.com/questions/70529682/create-pop-up-dialog-in-flutter
  /// Pop-up window allowing a user to change their password
  _editPasswordDialogBuilder() {
    showDialog (
        context: context,
        builder: (BuildContext context) {

          //Pop-up window
          return AlertDialog(
            title: const Text("Oppdater passord"),
            content: SingleChildScrollView(
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[

                  //Text input field for old password, to be used in re-authentication
                  Container(
                    padding: const EdgeInsets.all(8.0),
                    child: TextField(
                      obscureText: true,
                      controller: oldPasswordController,
                      onChanged: (String value) {
                        oldPassword = value;
                      },
                      decoration: const InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Gammelt passord'),
                    ),
                  ),

                  //Text input field for new password
                  Container(
                    padding: const EdgeInsets.all(8.0),
                    child: TextField(
                      obscureText: true,
                      controller: newPasswordController,
                      onChanged: (String value) {
                        newPassword = value;
                      },
                      decoration: const InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Nytt passord'),
                    ),
                  ),

                  //Text input field for repeating the new password
                  Container(
                    padding: const EdgeInsets.all(8.0),
                    child: TextField(
                      obscureText: true,
                      controller: repeatPasswordController,
                      onChanged: (String value) {
                        repeatPassword = value;
                      },
                      decoration: const InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Gjenta nytt passord'),
                    ),
                  ),

                  //Confirm update button
                  Container(
                    width: double.infinity,
                    height: 60,
                    padding: const EdgeInsets.all(8.0),
                    child: ElevatedButton(
                      onPressed: () {
                        int passCheck = validatePassword(newPassword);
                        if (passCheck == 0 && newPassword == repeatPassword) {
                          Future <int> update = firebaseUpdatePassword(oldPassword, newPassword);
                          update.whenComplete(() => Navigator.of(context).pop());
                        } else if (passCheck == 1) {
                          const snackBar = SnackBar(
                            content: Text('Passord må være lengre enn åtte tegn'),
                            backgroundColor: Colors.red,
                          );
                          ScaffoldMessenger.of(context).showSnackBar(snackBar);
                          Navigator.of(context).pop();
                        } else if (passCheck == 2) {
                          const snackBar = SnackBar(
                            content: Text('Passord må inneholde minst en bokstav'),
                            backgroundColor: Colors.red,
                          );
                          ScaffoldMessenger.of(context).showSnackBar(snackBar);
                          Navigator.of(context).pop();
                        } else if (passCheck == 3) {
                          const snackBar = SnackBar(
                            content: Text('Passordet må inneholde minst et tall'),
                            backgroundColor: Colors.red,
                          );
                          ScaffoldMessenger.of(context).showSnackBar(snackBar);
                          Navigator.of(context).pop();
                        } else if (newPassword != repeatPassword) {
                          const snackBar = SnackBar(
                            content: Text('Passordene er ikke det samme'),
                            backgroundColor: Colors.red,
                          );
                          ScaffoldMessenger.of(context).showSnackBar(snackBar);
                          Navigator.of(context).pop();
                        }
                        //Errors are not handled yet

                      },
                      child: const Text(
                        "Lagre",
                      ),
                    ),
                  ),
                ],
              ),
            ),
          );
        }
    );
  }

}