import 'package:billettapp_for_vitensenteret/towerOfHanoi/game_object.dart';
import 'package:flutter/foundation.dart';


/// Checks rules to see if a given piece [gameObject] is allowed to move on [gameBoard]
/// Returns true if the [gameObject] can be moved
bool checkIfPieceCanBeMoved(GameObject gameObject, List<String> gameBoard, int index,){
  // Looping through the gameboard
  for (int i = 0; i < gameBoard.length; i++){
    // Finding the piece
    if (gameBoard[i] == gameObject.color){
      // If the index of the desired new move is the same as the old one, return false
      if(i == index){
        return false;
      }
      // If it the same rod it originated from
      if(i != 0){
        // If it has a piece resting on it, edge-case for index 4 and 8
        if(gameBoard[i - 1] != 'empty' && i != 4 && i != 8){
          return false;
        }
      }
    }
  }
  return true;
}


/// Checks if the location the player is trying to place a [gameObject] on has anything under it on the [gameBoard]
/// Returns true if it can be placed
bool checkLegalMove(GameObject gameObject, List<String> gameBoard, int index, int lastIndex) {
  // If the piece can be moved
  if (!checkIfPieceCanBeMoved(gameObject, gameBoard, index)) {
    return false;
  }

  // Edge-case where its the bottom of the rod
  if (index == 3 || index == 7 || index == 11) {
    if (gameBoard[index] == 'empty') {
      gameObject.lastIndex = index;
      return true;
    }
  }

  // If the place you are trying to put the piece on is empty
  if (gameBoard[index] == 'empty') {
    // Check whats underneath
    int indexOfNextAvailableSpace = indexesToPieceUnderneath(gameBoard, index);
    // Check which piece is there, to determine if the current piece can be placed there
    if (canPieceBePlaced(gameObject, gameBoard, indexOfNextAvailableSpace)){
      gameObject.lastIndex = indexOfNextAvailableSpace;
      return true;
    }
  }
  return false;
}


/// Checks if the [gameBoard] list is the same as the [winBoard] list
/// Returns true if they match
bool checkWinCondition(List<String> gameBoard){
  List<String> winBoard = ['empty', 'empty','empty','empty','empty','empty','empty','empty','red', 'green', 'blue', 'yellow', 'empty'];
  return listEquals(winBoard, gameBoard);
}


/// Checks if the current [index] of [gameBoard] is occupied by a piece
/// Returns true if the current index is occupied
bool checkIfOccupied(List<String> gameBoard, int index){
  if(index > gameBoard.length){
    return false;
  }
  return gameBoard[index] != 'empty';
}


/// Checks the given [index] of [gameBoard] if a [gameObject] can be placed there based on the color of the piece
/// Returns true if it can be placed
bool canPieceBePlaced(GameObject gameObject, List<String> gameBoard, int index){
  // If the gameObject is red, it can always be placed
  if (gameObject.color == 'red'){
    switch(gameBoard[index + 1]){
      case 'green':
        return true;
      case 'blue':
        return true;
      case 'yellow':
        return true;
      case 'empty':
        return true;
    }
    return false;
  }
  // If the gameObject is green, it can be placed on anything except red
  if (gameObject.color == 'green'){
    switch(gameBoard[index + 1]){
      case 'red':
        return false;
      case 'blue':
        return true;
      case 'yellow':
        return true;
      case 'empty':
        return true;
    }
  }
  // Blue can only be placed on yellow
  if (gameObject.color == 'blue'){
    switch(gameBoard[index + 1]){
      case 'red':
        return false;
      case 'green':
        return false;
      case 'yellow':
        return true;
      case 'empty':
        return true;
    }
  }
  // Yellow can only be placed on empty
  if (gameObject.color == 'yellow' && gameBoard[index + 1] == 'empty'){
    return true;
  }
  return false;
}


/// Counts the number of empty spaces under the chosen [index] of [gameBoard]
/// Returns the number of empty places under the current index
int indexesToPieceUnderneath(List<String> gameBoard, int index){
  int i = index;
  for (i = index; i < gameBoard.length; i++){
    // If there is a piece, stop the count
    if(gameBoard[i + 1] != 'empty'){
      return i;
    }
    // If it reaches the bottom, return that index
    if (i == 3 || i == 7 || i == 11){
      return i;
    }
  }
  return i;
}


/// Gets the color id of the [gameObject] at the chosen [index] of [gameBoard]
int getId(List<String> gameBoard, int index){
  switch(gameBoard[index]){
    case 'red':
      return 0;
    case 'green':
      return 1;
    case'blue':
      return 2;
    case 'yellow':
      return 3;
  }
  return 0;
}

/// Takes a [tileId] and returns the string path of the image that should be displayed for the object
String getImage(int tileId){
  //If the tileId is in the top row, return top column image
  if(tileId == 0 || tileId == 4 || tileId == 8){
    return "images/rodTop.png";
  }
  //If the tileId is in the bottom row, return bottom column image
  if(tileId == 3 || tileId == 7 || tileId == 11){
    return "images/rodBottom.png";
  }
  return "images/rodCenter.png";
}

/// Returns a victory message string based on amount of [moves] player made
String victoryMessageBasedOnMoves(int moves){
  if(moves <= 15){
    return 'Gratulerer, du klarte det med minst antall trekk!';
  } else if (moves <= 30){
    return 'Gratulerer, du klarte det!';
  } else if (moves <= 45){
    return 'Gratulerer, du klarte det til slutt!';
  }
  return 'Imponerende mengde trekk. Du ga deg aldri, gratulerer!';
}