import 'dart:core';
import 'package:billettapp_for_vitensenteret/home_games.dart';
import 'package:billettapp_for_vitensenteret/towerOfHanoi/functions.dart';
import 'package:billettapp_for_vitensenteret/towerOfHanoi/game_object.dart';
import 'package:flutter/material.dart';
import 'package:animated_text_kit/animated_text_kit.dart';
import '../colors.dart';

/// Widget containing main game loop of Tower of Hanoi
class TowerOfHanoi extends StatefulWidget {
  const TowerOfHanoi({Key? key}) : super(key: key);

  @override
  _GameState createState() => _GameState();
}

GlobalKey<ScaffoldState> scaffoldKey = GlobalKey();

// Variables for game logic
List<String> gameBoard = [
  'red',
  'green',
  'blue',
  'yellow',
  'empty',
  'empty',
  'empty',
  'empty',
  'empty',
  'empty',
  'empty',
  'empty',
  'empty'
];
int lastIndex = 0;
int lastPiece = 0;
int moves = 0;
bool win = false;

// Rod images
String tileTop = "assets/images/rodTop.png";
String tileCenter = "assets/images/rodCenter.png";
String tileBottom = "assets/images/rodBottom.png";

// Game objects
final List<GameObject> _gameObjects = [
  GameObject(
    color: "red",
    lastIndex: 0,
    imageId: "assets/images/red.png",
  ),
  GameObject(
    color: "green",
    lastIndex: 1,
    imageId: "assets/images/green.png",
  ),
  GameObject(
    color: "blue",
    lastIndex: 2,
    imageId: "assets/images/blue.png",
  ),
  GameObject(
    color: "yellow",
    lastIndex: 3,
    imageId: "assets/images/yellow.png",
  ),
];

class _GameState extends State<TowerOfHanoi> {
  // Game description
  final String gameDescription =
      "1. Målet er å flytte alle brikkene til staven på høyre side.\n"
      "2. Du kan ikke sette en større brikke på en mindre brikke.\n"
      "3. Du kan ikke flytte brikker som er under en annen brikke.\n\n"
      "                                            Lykke til!";

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
          //Top bar
          appBar: AppBar(
            leading: BackButton(
              onPressed: () => Navigator.push(
                context,
                MaterialPageRoute(builder: (context) => const GameMenuPage()),
              ),
            ),
            title: const Text('Hanois Tårn'),
            titleSpacing: 00.0,
            centerTitle: true,
            toolbarHeight: 60.2,
            toolbarOpacity: 0.8,
            shape: const RoundedRectangleBorder(
              borderRadius: BorderRadius.only(
                  bottomRight: Radius.circular(25),
                  bottomLeft: Radius.circular(25)),
            ),
            elevation: 0.00,
            backgroundColor: morkBlaa,
          ),
          // Game window
          body: Column(
            children: [
              // Top menu bar
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  // Rules
                  TextButton(
                    onPressed: () => showDialog<String>(
                      context: context,
                      builder: (BuildContext context) => AlertDialog(
                        title: const Text('Hvordan spille Hanois tårn'),
                        content: Text(gameDescription),
                        actions: const <Widget>[],
                      ),
                    ),
                    style: TextButton.styleFrom(
                      foregroundColor: morkBlaa,
                    ),
                    child: const Text('Regler'),
                  ),
                  SizedBox(
                    width: MediaQuery.of(context).size.width / 10,
                  ),
                  // Restart
                  TextButton(
                    style: TextButton.styleFrom(
                      foregroundColor: morkBlaa,
                    ),
                    onPressed: () {
                      setState(() {
                        gameBoard = [
                          'red',
                          'green',
                          'blue',
                          'yellow',
                          'empty',
                          'empty',
                          'empty',
                          'empty',
                          'empty',
                          'empty',
                          'empty',
                          'empty',
                          'empty'
                        ];
                        moves = 0;
                        win = false;
                      });
                    },
                    child: const Text("Start på nytt"),
                  ),
                ],
              ),
              const SizedBox(
                height: 20,
              ),
              // Move counter window
              Center(
                child: Container(
                  height: MediaQuery.of(context).size.height / 10,
                  width: MediaQuery.of(context).size.width / 5,
                  decoration: const BoxDecoration(color: blaa, boxShadow: [
                    BoxShadow(
                        color: Colors.grey,
                        blurRadius: 2.0,
                        offset: Offset(2.0, 2.0))
                  ]),
                  child: const Center(
                    child: Text(
                      "Trekk:",
                      textAlign: TextAlign.center,
                      style: TextStyle(fontSize: 20.0),
                    ),
                  ),
                ),
              ),
              Center(
                child: Container(
                  height: MediaQuery.of(context).size.height / 10,
                  width: MediaQuery.of(context).size.width / 5,
                  decoration: const BoxDecoration(color: blaa, boxShadow: [
                    BoxShadow(
                        color: Colors.grey,
                        blurRadius: 2.0,
                        offset: Offset(2.0, 2.0))
                  ]),
                  child: Center(child: _scale(moves)),
                ),
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  SizedBox(
                    width: MediaQuery.of(context).size.width / 10,
                  ),
                  // Left tower
                  Column(
                    children: [
                      const SizedBox(
                        height: 20,
                      ),

                      //Note that the following code is structured with identical widgets however with
                      //different tileid's. Multiple attempts were made trying to optimize this code,
                      //However since this widget is a movable object with a target on top of it,
                      //we were not able to find a solution that worked for the whole grid and resolved
                      //to leave it as is for ease of understanding

                      //This is the gameboard, consisting of a grid of movable objects placed in a
                      // 4x3 grid.

                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileTop),
                                if (checkIfOccupied(gameBoard, 0))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 0)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 0)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 0;
                                      lastPiece = getId(gameBoard, 0);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 0)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 0, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileCenter),
                                if (checkIfOccupied(gameBoard, 1))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 1)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 1)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 1;
                                      lastPiece = getId(gameBoard, 1);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 1)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 1, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileCenter),
                                if (checkIfOccupied(gameBoard, 2))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 2)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 2)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 2;
                                      lastPiece = getId(gameBoard, 2);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 2)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 2, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileBottom),
                                if (checkIfOccupied(gameBoard, 3))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 3)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 3)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 3;
                                      lastPiece = getId(gameBoard, 3);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 3)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 3, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                    ],
                  ),
                  // Middle tower
                  Column(
                    children: [
                      const SizedBox(
                        height: 20,
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileTop),
                                if (checkIfOccupied(gameBoard, 4))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 4)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 4)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 4;
                                      lastPiece = getId(gameBoard, 4);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 4)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 4, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileCenter),
                                if (checkIfOccupied(gameBoard, 5))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 5)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 5)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 5;
                                      lastPiece = getId(gameBoard, 5);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 5)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 5, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileCenter),
                                if (checkIfOccupied(gameBoard, 6))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 6)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 6)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 6;
                                      lastPiece = getId(gameBoard, 6);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 6)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 6, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileBottom),
                                if (checkIfOccupied(gameBoard, 7))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 7)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 7)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 7;
                                      lastPiece = getId(gameBoard, 7);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 7)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 7, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                    ],
                  ),
                  // Right tower
                  Column(
                    children: [
                      const SizedBox(
                        height: 20,
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileTop),
                                if (checkIfOccupied(gameBoard, 8))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 8)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 8)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 8;
                                      lastPiece = getId(gameBoard, 8);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 8)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 8, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileCenter),
                                if (checkIfOccupied(gameBoard, 9))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 9)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 9)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 9;
                                      lastPiece = getId(gameBoard, 9);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 9)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(
                              _gameObjects[lastPiece], gameBoard, 9, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileCenter),
                                if (checkIfOccupied(gameBoard, 10))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 10)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 10)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 10;
                                      lastPiece = getId(gameBoard, 10);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 10)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(_gameObjects[lastPiece],
                              gameBoard, 10, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                      DragTarget<GameObject>(
                        builder: (
                          BuildContext context,
                          List<dynamic> accepted,
                          List<dynamic> rejected,
                        ) {
                          return SizedBox(
                            height: MediaQuery.of(context).size.height / 10,
                            width: MediaQuery.of(context).size.width / 6,
                            child: Stack(
                              children: [
                                Image.asset(tileBottom),
                                if (checkIfOccupied(gameBoard, 11))
                                  Draggable<GameObject>(
                                    data: _gameObjects[getId(gameBoard, 11)],
                                    feedback: Image.asset(
                                        _gameObjects[getId(gameBoard, 11)]
                                            .imageId),
                                    childWhenDragging: Container(),
                                    onDragStarted: () {
                                      lastIndex = 11;
                                      lastPiece = getId(gameBoard, 11);
                                    },
                                    onDragCompleted: () {
                                      setState(() {
                                        moves++;
                                        win = checkWinCondition(gameBoard);
                                      });
                                    },
                                    child: Image.asset(
                                        _gameObjects[getId(gameBoard, 11)]
                                            .imageId),
                                  ),
                              ],
                            ),
                          );
                        },
                        onWillAccept: (gameObject) {
                          return checkLegalMove(_gameObjects[lastPiece],
                              gameBoard, 11, lastIndex);
                        },
                        onAccept: (gameObject) {
                          setState(() {
                            //update new place
                            gameBoard[_gameObjects[lastPiece].lastIndex] =
                                _gameObjects[lastPiece].color;
                            //remove the piece from where it was
                            gameBoard[lastIndex] = 'empty';
                          });
                        },
                      ),
                    ],
                  ),
                ],
              ),
              //This is the end of the gameboard or the grid holding all the gameobjects.

              const SizedBox(
                height: 20,
              ),
              // Display victory widget
              if (win)
                Column(
                  children: [
                    Container(
                      height: 70.0,
                      width: 650.0,
                      decoration: const BoxDecoration(color: blaa, boxShadow: [
                        BoxShadow(
                            color: Colors.grey,
                            blurRadius: 2.0,
                            offset: Offset(2.0, 2.0))
                      ]),
                      child: Center(child: _victory(moves)),
                    ),
                    TextButton(
                      style: TextButton.styleFrom(
                        foregroundColor: morkBlaa,
                      ),
                      onPressed: () {
                        setState(() {
                          gameBoard = [
                            'red',
                            'green',
                            'blue',
                            'yellow',
                            'empty',
                            'empty',
                            'empty',
                            'empty',
                            'empty',
                            'empty',
                            'empty',
                            'empty',
                            'empty'
                          ];
                          moves = 0;
                          win = false;
                        });
                      },
                      child: const Text("Start på nytt"),
                    ),
                  ],
                ),
            ],
          )),
    );
  }
}

// Widget for the score animation
// From https://pub.dev/packages/animated_text_kit
Widget _scale(int moves) {
  return SizedBox(
    child: DefaultTextStyle(
      style: const TextStyle(
        fontSize: 20.0,
        color: morkBlaa,
      ),
      child: Center(
        child: AnimatedTextKit(
          repeatForever: true,
          animatedTexts: [
            ScaleAnimatedText('$moves'),
          ],
        ),
      ),
    ),
  );
}

// Widget for the victory screen text animation
// From https://pub.dev/packages/animated_text_kit
Widget _victory(int moves) {
  return DefaultTextStyle(
    style: const TextStyle(
      fontSize: 25.0,
      color: morkBlaa,
    ),
    child: AnimatedTextKit(
      animatedTexts: [
        WavyAnimatedText(victoryMessageBasedOnMoves(moves),
            speed: const Duration(milliseconds: 200)),
      ],
      isRepeatingAnimation: true,
      repeatForever: true,
    ),
  );
}
